let rootUrl = 'http://localhost:8080/';
let dmnApi = rootUrl + 'api/dmn/';

/**
 * Global definition for verification results
 * @type {VerifierResultSet}
 */
let verifierResults = {};

function cleanDmnVerifierRoot() {
  let $root = $('#root-dmn-verifier').empty();
  let $header = $(`      
      <div id="dmn-verifier-header">
  `);
  $header.append($(`
    <button class="clickable dmn-verifier-header-item" id="dmn-button-verify" onClick="checkVerifications()">
    Verify</button>
  `));
  $root.append($header);
  $root.append($(`<div id="dmn-verifier-content">`));
  return $root;
}

function checkVerifications() {
  cleanDmnVerifierRoot();
  getVerifications();
}

function getVerifications() {
  dmnModeler.saveXML({format: true}, function (err, xml) {
    if (err) {
      return console.log(
          'There is an error in requesting the dmn verifications.. ' + err);
    } else {
      console.log('Requesting dmn verifications..');
    }
    $.ajax({
      url: dmnApi + 'verification',
      type: 'POST',
      contentType: 'text/xml',
      data: xml,

      success: function (data) {
        console.log('successful dmn verification request:');
        console.log(data);
        verifierResults = data;
        // sort the list 1. classification, 2. size of entries
        verifierResults.verifier.sort(function (
            /** VerifierResult */ firstEl, /** VerifierResult */ secondEl) {
          if (firstEl.type.classification.name
              === secondEl.type.classification.name) {
            if (firstEl.size === secondEl.size) {
              return firstEl.type.name < secondEl.type.name ? -1 : 1;
            } else if (firstEl.size < secondEl.size) {
              return 1;
            } else {
              return -1;
            }
          } else if (firstEl.type.classification.name
              < secondEl.type.classification.name) {
            return -1;
          } else {
            return 1;
          }
        });
        renderDmnVerifierOptions();
      }
    });
  });
}

function renderDmnVerifierOptions() {
  let $header = $('#dmn-verifier-header');
  let $select = $(`
    <select name="verifier" id="verifier"
      class="dmn-verifier-header-item dmn-verifier-select clickable"
      onchange="renderVerifierResult(this)">
  `);
  $select.append($(`<option>Select a verifier</option>`));
  let currentOpt = '';
  let $curGroup;
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    let bolNewGroup = currentOpt
        !== verifierResults.verifier[i].type.classification.niceName;
    if (bolNewGroup) {
      if (currentOpt === currentOpt) {
        $select.append($curGroup);
      }
      $curGroup = ($(
          `<optgroup label="${verifierResults.verifier[i].type.classification.niceName}">
      `));
      currentOpt = verifierResults.verifier[i].type.classification.niceName;
    }
    $curGroup.append(renderSelectEntry(verifierResults.verifier[i]));
  }
  $select.append($curGroup);
  $header.append($select);
}

/**
 *
 * @param verifier
 * @type VerifierResult
 */
function renderSelectEntry(verifier) {
  return $(`
      <option value="${verifier.type.name}">${verifier.type.niceName} (${verifier.size})</option>
    `);
}

/**
 *
 * @param verifierSelect
 * @Type option
 */
function renderVerifierResult(verifierSelect) {
  let verifier = findVerifierByName(verifierSelect.value);
  let $verifierEntries = $('#dmn-verifier-content');
  $verifierEntries.empty();
  if (verifier !== undefined) {
    $verifierEntries.append($(`
    <div>${verifier.type.niceName}: ${verifier.type.description}</div>
  `));
    let $entryList = $(`<ul>`);
    $verifierEntries.append($entryList);
    for (let i = 0; i < verifier.entries.length; i++) {
      $entryList.append(renderVerificationEntry(verifier.entries[i]));
    }
  }
}

/**
 *
 * @param verificationEntry
 * @Type VerificationEntry
 */
function renderVerificationEntry(verificationEntry) {
  console.log(verificationEntry);
  return $(`
    <li class="dmn-${verificationEntry.verificationClassification}">${verificationEntry.message}</li>
  `);
}

/**
 *
 * @param verifierName
 * @type string
 * @return VerifierResult
 */
function findVerifierByName(verifierName) {
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    if (verifierResults.verifier[i].type.name === verifierName) {
      return verifierResults.verifier[i];
    }
  }
  return undefined;
}
